﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace testReflection
{
    public delegate void TestDelegate(string info);
    class Program
    {
        static void Main(string[] args)
        {
            //从Dll中加载
            //Assembly ass = Assembly.LoadFile(@"TestReflect.dll");
            //Type myType = ass.GetType("testReflection.Person");
            //object aPerson = ass.CreateInstance("Person");

            //取得类型
            Type myType = Type.GetType("testReflection.Person");
            //构造函数要用到的参数
            object[] constuctParms = new object[] { "Brad Pitt" };
            //创建实例
            //object TestName = Assembly.GetAssembly(myType).CreateInstance("Person"); 
            object aPerson = Activator.CreateInstance(myType, constuctParms);

            //使用MethodInfo 和Invoke 调用方法
            MethodInfo displayInfoMethod = myType.GetMethod("DisplayInfo");
            displayInfoMethod.Invoke(aPerson, new object[] { "Using Invoke to call Method DisplayInfo()" });

            //使用InvokeMember 调用方法
            //调用方法的一些标志位
            BindingFlags flag = BindingFlags.InvokeMethod | BindingFlags.Public | BindingFlags.Instance;
            myType.InvokeMember("DisplayInfo", flag, null, aPerson, new object[] { "Using InvokeMethod to call DisplayInfo()" });
            //如果方法有返回值
            string name = (string)myType.InvokeMember("getName", flag, null, aPerson, null);
            System.Console.WriteLine("call getName(), return: " + name);

            //设置属性值
            myType.InvokeMember("Age", BindingFlags.SetProperty, null, aPerson, new object[] { 30 });
            //得到属性值
            int age = (int)myType.InvokeMember("Age", BindingFlags.GetProperty, null, aPerson, null);
            System.Console.WriteLine("Get the property of Age : " + Convert.ToString(age));

            //设置字段值
            myType.InvokeMember("Name", BindingFlags.SetField, null, aPerson, new object[] { "Michal Jodn" });
            //获取字段值
            string fname = (string)myType.InvokeMember("Name", BindingFlags.GetField, null, aPerson, null);
            System.Console.WriteLine("Get the Field Value of Name : " + fname);

            myType.InvokeMember("DisplayName", flag, null, aPerson, null);

            //获得方法集
            MethodInfo[] methods = myType.GetMethods();
            foreach (MethodInfo m in methods)
            {
                System.Console.WriteLine(m.Name);
            }
            //同样还有：GetFiedls()、GetProperties()、GetEvents()等方法

            //使用Delegate
            //此方法是静态的，所以必须提供委托类型。
            TestDelegate dg = (TestDelegate)Delegate.CreateDelegate(typeof(testReflection.TestDelegate), aPerson, "DisplayInfo");
            dg("Test Delegate by call DisplayInfo()");


            //获得解决方案的所有Assembly
            Assembly[] AX = AppDomain.CurrentDomain.GetAssemblies();
            //遍历显示每个Assembly的名字
            foreach (object var in AX)
            {
                Console.WriteLine("Assembly的名字:" + var.ToString());
            }
            //使用一个已知的Assembly名称,来创建一个Assembly
            //通过CodeBase属性显示最初指定的程序集的位置
            Console.WriteLine("最初指定的程序集TestReflection的位置:" + Assembly.Load("TestReflection").CodeBase);
            System.Console.ReadLine();
        }
    }

    public class Person
    {
        public string Name;
        private int _Age;
        public int Age
        {
            get
            {
                return _Age;
            }
            set
            {
                _Age = value;
            }
        }
        public Person(string Name)
        {
            this.Name = Name;
        }
        public void DisplayInfo(string info)
        {
            System.Console.WriteLine(info);
            System.Console.WriteLine("called sucessfully!");
        }
        public void DisplayName()
        {
            System.Console.WriteLine(Name);
        }
        public string getName()
        {
            return Name;
        }
    }
}


